// -*- c++ -*-
// $Id$
/*
 * Copyright 2005-2007 Spencer Olson
 *
 * $Log$
 *
 */

/** \file
 * Test the various Laser beams that are currently possible in the library.
 */

#define DISABLE_SHELL_LOOKUP
#define TESTLASER 1

#include <fstream>
#include <olson-tools/physical.h>
#include <olson-tools/strutil.h>
#include <laser-cooling/OpticalForce.h>

using namespace physical::constants;
using namespace physical::units;
using LaserCooling::OpticalForce;

#include "../common/selectlaser.h"

int main() {
    LaserForce laserForce;

    laserForce.Gravity::bg = 0.0;
    laserForce.super0::mass = 87*amu;
    initLaser(laserForce);

    printLaserInfo(laserForce, std::cout, "");
    std::cout << "xi : dx : xf  =  "
              << tr.xi << " : " << tr.dx << " : " << tr.xf << '\n'
              << "yi : yx : yf  =  "
              << tr.yi << " : " << tr.dy << " : " << tr.yf << '\n'
              << "zi : zx : zf  =  "
              << tr.zi << " : " << tr.dz << " : " << tr.zf << std::endl;

#ifdef LFILE
    std::ofstream ofile("laserfield.dat");

    ofile << "# r[X..Z] a[X..Z] V" << std::endl;
#endif

    int nX = 1 + (tr.xf - tr.xi)/tr.dx,
        nY = 1 + (tr.yf - tr.yi)/tr.dy;

    std::cout << "(nx, ny) = " << "(" << nX << "," << nY << ")" << std::endl;

    Vector<double,3> r;
    int file_num=0;
    for (        r[Z] = tr.zi; r[Z] <= tr.zf; r[Z] += tr.dz, file_num++) {
        std::ofstream ifile(("int_"+to_string(file_num)+".pgm").c_str());
        ifile << "P5\n" << nX << " " << nY << "\n255\n";
        for (    r[Y] = tr.yi; r[Y] <= tr.yf; r[Y] += tr.dy) {
            for (r[X] = tr.xi; r[X] <= tr.xf; r[X] += tr.dx) {
                ifile << (uint8_t)(totalIntensity(laserForce,r)*255.0/tr.Imax);
#ifdef LFILE
                Vector<double,3> a;
                laserForce.accel(a,r);
                ofile << r << '\t'
                      << a << '\t'
                      << laserForce.potential(r) << '\t'
                      << laserForce.laser(r) << '\t'
                      << laserForce.gamma_p(r) << '\t'
#if 0
                      << laserForce.laser.w(r) << '\t'
                      << laserForce.laser.rho2(r) << '\t'
                      << laserForce.laser.z(r) << '\t'
                      << laserForce.laser.E(r) << '\t'
#endif
                      << '\n';
#endif
            }
#ifdef LFILE
            ofile << '\n';
#endif
        }
#ifdef LFILE
        ofile << '\n';
#endif
    }

#ifdef LFILE
    ofile.close();
#endif
}
