
#ifndef CROSSED_HOLLOW_H
#define CROSSED_HOLLOW_H

//#define MULTILEVEL_ATOM

#include <olson-tools/Vector.h>
#include <dsmc/BoundaryObject.h>
#include <laser-cooling/OpticalForce.h>
#include <laser-cooling/SpontScatMon.h>
#include <laser-cooling/ScaleIntensity.h>

//typedef Particle Particle_t;
typedef StateParticle Particle_t;


using LaserCooling::LaguerreGaussianLaser;
using LaserCooling::OpticalForce;
using LaserCooling::LaserFieldLookup;
using LaserCooling::IntensityRecord;
#ifdef SPONTSCATMON_H
using LaserCooling::SpontScatMonitor;
#endif
using LaserCooling::ScaleIntensity;
using LaserCooling::GaussianLaser;

typedef OpticalForce<
    ScaleIntensity< LaguerreGaussianLaser >,
    0
#ifdef SPONTSCATMON_H
    ,SpontScatMonitor
#endif
> beam_x;

typedef OpticalForce<
    ScaleIntensity< LaguerreGaussianLaser >,
    1
#ifdef SPONTSCATMON_H
    ,SpontScatMonitor
#endif
> beam_y;

//#define USEMOLASSES

#ifdef USEMOLASSES
typedef OpticalForce< GaussianLaser, 0 > beam_x0;
typedef OpticalForce< GaussianLaser, 1 > beam_x1;
typedef OpticalForce< GaussianLaser, 2 > beam_y0;
typedef OpticalForce< GaussianLaser, 3 > beam_y1;
typedef OpticalForce< GaussianLaser, 4 > beam_z0;
typedef OpticalForce< GaussianLaser, 5 > beam_z1;

typedef AddForce < beam_x0, beam_x1 > xpair;
typedef AddForce < beam_y0, beam_y1 > ypair;
typedef AddForce < beam_z0, beam_z1 > zpair;

typedef AddForce< AddForce< xpair, ypair >, zpair > Molasses;
void initMolasses(Molasses & molasses);
#endif


typedef AddForce<
#ifdef USEMOLASSES
    AddForce <
#endif
        AddForce <beam_x, beam_y>,
#ifdef USEMOLASSES
        Molasses >,
#endif
    Gravity
> LaserForce;

const double detuning = 0,//+100*GHz,
             lambda   = 779.244 * nm,
             power    = 0.5 * 134.0*mW,
             w0       = 70.*um,
#ifdef MULTILEVEL_ATOM
             g_p      = 5e-5 /* s^(-1) / (W/m^2) : 779.244nm */,
#endif
             g23      = 0.233752 /* 779.244nm */,
             g32      = 0.166843 /* 779.244nm */;

void initLaser(LaserForce & laserForce) {
#ifdef USEMOLASSES
    initMolasses(laserForce);
#endif
    /* set the laser timing. */
    laserForce.beam_x::laser.t_i = 0.0;
    laserForce.beam_y::laser.t_i = 0.0;

    laserForce.beam_x::laser.t_f = 5.*ms;
    laserForce.beam_y::laser.t_f = 5.*ms;

#ifdef TESTLASER
    laserForce.beam_x::laser.set_scale(1.0);
    laserForce.beam_y::laser.set_scale(1.0);
#else
    laserForce.beam_x::laser.set_scale(0.0);
    laserForce.beam_y::laser.set_scale(0.0);
#endif


    laserForce.beam_x::laser.k  = V3(1,0,0);
    laserForce.beam_x::laser.lambda = lambda;
    laserForce.beam_x::detuning     = detuning;
    laserForce.beam_x::laser.P_0    = power;
    laserForce.beam_x::laser.setCollimation(w0);
    laserForce.beam_x::laser.l = 4;
    laserForce.beam_x::laser.p = 0;
#ifdef MULTILEVEL_ATOM
    laserForce.beam_x::g_p     = g_p;
#endif
    laserForce.beam_x::spontScatMonitor.g23 = g23;
    laserForce.beam_x::spontScatMonitor.g32 = g32;
    laserForce.beam_x::initialize();

    laserForce.beam_y::laser.k  = V3(0,1,0);
    laserForce.beam_y::laser.lambda = lambda;
    laserForce.beam_y::detuning     = detuning;
    laserForce.beam_y::laser.P_0    = power;
    laserForce.beam_y::laser.setCollimation(w0);
    laserForce.beam_y::laser.l = 4;
    laserForce.beam_y::laser.p = 0;
#ifdef MULTILEVEL_ATOM
    laserForce.beam_y::g_p     = g_p;
#endif
    laserForce.beam_y::spontScatMonitor.g23 = g23;
    laserForce.beam_y::spontScatMonitor.g32 = g32;
    laserForce.beam_y::initialize();
}

double totalIntensity(const LaserForce & laserForce, const Vector<double,3> & r) {
    return laserForce.beam_y::laser(r) + laserForce.beam_x::laser(r);
}

std::ostream & printLaserInfo(const LaserForce & laserForce, std::ostream & out, const std::string & prefix = "") {
    out << prefix << "beam x:" << '\n'
        << prefix << "w0:  "            << laserForce.beam_x::laser.w0 << '\n'
        << prefix << "zR:  "            << laserForce.beam_x::laser.zR << '\n'
        << prefix << "laser.lambda:  "  << laserForce.beam_x::laser.lambda << '\n'
        << prefix << "detuning:  "      << laserForce.beam_x::detuning << '\n'

        << prefix << "beam y:" << '\n'
        << prefix << "w0:  "            << laserForce.beam_y::laser.w0 << '\n'
        << prefix << "zR:  "            << laserForce.beam_y::laser.zR << '\n'
        << prefix << "laser.lambda:  "  << laserForce.beam_y::laser.lambda << '\n'
        << prefix << "detuning:  "      << laserForce.beam_y::detuning << std::endl;

    return out;
}

double initial_particle_sigma_x(const LaserForce & laserForce) {
    return w0;
}

void set_laser_timing(LaserForce & laserForce, const double & t) {
    laserForce.beam_x::laser.set_scale(t);
    laserForce.beam_y::laser.set_scale(t);
}

static const struct {
/* LASER TEST RANGES */
static const double
    xi = -2*w0,
    xf =  2.000001*w0,
    dx = .05*w0,

    yi = -2*w0,
    yf =  2.000001*w0,
    dy = .05*w0,

    zi = -2*w0,
    zf =  2.000001*w0,
    dz = .05*w0;

static const double Imax = 2e6; /* W/m^2 */
} tr;

const Vector<double,3> camera_min(V3(-100.*um, -100*um, -100.*um));
const Vector<double,3> camera_max(V3( 100.*um,  100*um,  100.*um));



/* ************ BEGIN DSMC BOUNDARIES ***************** */

const double Len = 5.0*w0;

typedef octree::VacuumBoundaryObject<Particle_t>     vac_type;
/** Vacuum boundary conditions. */
const int num_vacuo = 6;
vac_type vacuo[num_vacuo] = {
    /*        normal     point            side-length    scaling */
    vac_type(V3(-1,0,0), V3( 0.5*Len,0,0), V3(0,DBL_MAX,DBL_MAX), 0.5*w0),
    vac_type(V3( 1,0,0), V3(-0.5*Len,0,0), V3(0,DBL_MAX,DBL_MAX), 0.5*w0),
    vac_type(V3(0,-1,0), V3(0, 0.5*Len,0), V3(DBL_MAX,0,DBL_MAX), 0.5*w0),
    vac_type(V3(0, 1,0), V3(0,-0.5*Len,0), V3(DBL_MAX,0,DBL_MAX), 0.5*w0),
    vac_type(V3(0,0,-1), V3(0,0, 0.5*Len), V3(DBL_MAX,DBL_MAX,0), 0.5*w0),
    vac_type(V3(0,0, 1), V3(0,0,-0.5*Len), V3(DBL_MAX,DBL_MAX,0), 0.5*w0)
};

template <class DSMC>
void addBoundaries(DSMC & dsmc) {
    for (int i = 0; i < num_vacuo; i++) {
        dsmc.addBoundaryObject(&vacuo[i]);
    }
}

/* ************** END DSMC BOUNDARIES ***************** */


#ifdef USEMOLASSES
void initMolasses(Molasses & molasses) {
    const double detuning = -60*MHz,
                 lambda   = 780.244 * nm,
                 power    = 0.01*mW,
                 w0       = 1*cm;

    molasses.beam_x0::laser.k      = V3(1,0,0);
    molasses.beam_x0::laser.lambda = lambda;
    molasses.beam_x0::detuning     = detuning;
    molasses.beam_x0::laser.setPower(power);
    molasses.beam_x0::laser.setCollimation(w0);
    molasses.beam_x0::initialize();

    molasses.beam_x1::laser.k      = V3(-1,0,0);
    molasses.beam_x1::laser.lambda = lambda;
    molasses.beam_x1::detuning     = detuning;
    molasses.beam_x1::laser.setPower(power);
    molasses.beam_x1::laser.setCollimation(w0);
    molasses.beam_x1::initialize();

    molasses.beam_y0::laser.k      = V3(0,1,0);
    molasses.beam_y0::laser.lambda = lambda;
    molasses.beam_y0::detuning     = detuning;
    molasses.beam_y0::laser.setPower(power);
    molasses.beam_y0::laser.setCollimation(w0);
    molasses.beam_y0::initialize();

    molasses.beam_y1::laser.k      = V3(0,-1,0);
    molasses.beam_y1::laser.lambda = lambda;
    molasses.beam_y1::detuning     = detuning;
    molasses.beam_y1::laser.setPower(power);
    molasses.beam_y1::laser.setCollimation(w0);
    molasses.beam_y1::initialize();

    molasses.beam_z0::laser.k      = V3(0,0,1);
    molasses.beam_z0::laser.lambda = lambda;
    molasses.beam_z0::detuning     = detuning;
    molasses.beam_z0::laser.setPower(power);
    molasses.beam_z0::laser.setCollimation(w0);
    molasses.beam_z0::initialize();

    molasses.beam_z1::laser.k      = V3(0,0,-1);
    molasses.beam_z1::laser.lambda = lambda;
    molasses.beam_z1::detuning     = detuning;
    molasses.beam_z1::laser.setPower(power);
    molasses.beam_z1::laser.setCollimation(w0);
    molasses.beam_z1::initialize();
}
#endif

#endif // CROSSED_HOLLOW_H
