
#ifndef MOLASSES_H
#define MOLASSES_H

using LaserCooling::GaussianLaser;

typedef OpticalForce< GaussianLaser, 0 > beam_x0;
typedef OpticalForce< GaussianLaser, 1 > beam_x1;
typedef OpticalForce< GaussianLaser, 2 > beam_y0;
typedef OpticalForce< GaussianLaser, 3 > beam_y1;
typedef OpticalForce< GaussianLaser, 4 > beam_z0;
typedef OpticalForce< GaussianLaser, 5 > beam_z1;

typedef AddForce < beam_x0, beam_x1 > xpair;
typedef AddForce < beam_y0, beam_y1 > ypair;
typedef AddForce < beam_z0, beam_z1 > zpair;

typedef AddForce<
    AddForce<
        AddForce< xpair, ypair >,
        zpair
    >, 
    Gravity
> LaserForce;

void initLaser(LaserForce & laserForce) {
    const double detuning = -18*MHz,
                 lambda   = 780.244 * nm,
                 power    = 30.0*mW,
                 w0       = 1*cm;

    laserForce.beam_x0::laser.k      = V3(1,0,0);
    laserForce.beam_x0::laser.lambda = lambda;
    laserForce.beam_x0::detuning     = detuning;
    laserForce.beam_x0::laser.setPower(power);
    laserForce.beam_x0::laser.setCollimation(w0);
    laserForce.beam_x0::initialize();

    laserForce.beam_x1::laser.k      = V3(-1,0,0);
    laserForce.beam_x1::laser.lambda = lambda;
    laserForce.beam_x1::detuning     = detuning;
    laserForce.beam_x1::laser.setPower(power);
    laserForce.beam_x1::laser.setCollimation(w0);
    laserForce.beam_x1::initialize();

    laserForce.beam_y0::laser.k      = V3(0,1,0);
    laserForce.beam_y0::laser.lambda = lambda;
    laserForce.beam_y0::detuning     = detuning;
    laserForce.beam_y0::laser.setPower(power);
    laserForce.beam_y0::laser.setCollimation(w0);
    laserForce.beam_y0::initialize();

    laserForce.beam_y1::laser.k      = V3(0,-1,0);
    laserForce.beam_y1::laser.lambda = lambda;
    laserForce.beam_y1::detuning     = detuning;
    laserForce.beam_y1::laser.setPower(power);
    laserForce.beam_y1::laser.setCollimation(w0);
    laserForce.beam_y1::initialize();

    laserForce.beam_z0::laser.k      = V3(0,0,1);
    laserForce.beam_z0::laser.lambda = lambda;
    laserForce.beam_z0::detuning     = detuning;
    laserForce.beam_z0::laser.setPower(power);
    laserForce.beam_z0::laser.setCollimation(w0);
    laserForce.beam_z0::initialize();

    laserForce.beam_z1::laser.k      = V3(0,0,-1);
    laserForce.beam_z1::laser.lambda = lambda;
    laserForce.beam_z1::detuning     = detuning;
    laserForce.beam_z1::laser.setPower(power);
    laserForce.beam_z1::laser.setCollimation(w0);
    laserForce.beam_z1::initialize();
}

std::ostream & printLaserInfo(const LaserForce & laserForce, std::ostream & out, const std::string & prefix = "") {
    out
        << prefix << "+x :"
        << prefix << "           w0:  " << laserForce.beam_x0::laser.w0 << '\n'
        << prefix << "           zR:  " << laserForce.beam_x0::laser.zR << '\n'
        << prefix << "     detuning:  " << laserForce.beam_x0::detuning << '\n'

        << prefix << "-x :"
        << prefix << "           w0:  " << laserForce.beam_x1::laser.w0 << '\n'
        << prefix << "           zR:  " << laserForce.beam_x1::laser.zR << '\n'
        << prefix << "     detuning:  " << laserForce.beam_x1::detuning << '\n'

        << prefix << "+y :"
        << prefix << "           w0:  " << laserForce.beam_y0::laser.w0 << '\n'
        << prefix << "           zR:  " << laserForce.beam_y0::laser.zR << '\n'
        << prefix << "     detuning:  " << laserForce.beam_y0::detuning << '\n'

        << prefix << "-y :"
        << prefix << "           w0:  " << laserForce.beam_y1::laser.w0 << '\n'
        << prefix << "           zR:  " << laserForce.beam_y1::laser.zR << '\n'
        << prefix << "     detuning:  " << laserForce.beam_y1::detuning << '\n'

        << prefix << "+z :"
        << prefix << "           w0:  " << laserForce.beam_z0::laser.w0 << '\n'
        << prefix << "           zR:  " << laserForce.beam_z0::laser.zR << '\n'
        << prefix << "     detuning:  " << laserForce.beam_x0::detuning << '\n'

        << prefix << "-z :"
        << prefix << "           w0:  " << laserForce.beam_z1::laser.w0 << '\n'
        << prefix << "           zR:  " << laserForce.beam_z1::laser.zR << '\n'
        << prefix << "     detuning:  " << laserForce.beam_z1::detuning << '\n';

    return out << std::flush;
}

double initial_particle_sigma_x(const LaserForce & laserForce) {
    return laserForce.beam_x0::laser.w0;
}

#endif // MOLASSES_H
