
#ifndef SINGLE_HOLLOW_H
#define SINGLE_HOLLOW_H

using LaserCooling::LaguerreGaussianLaser;

typedef OpticalForce< LaguerreGaussianLaser > beam;

typedef AddForce< beam, Gravity > LaserForce;

void initLaser(LaserForce & laserForce) {
    const double detuning = +100*GHz,
                 lambda   = 780.244 * nm,
                 power    = 500.0*mW,
                 w0       = 0.2*mm;


    laserForce.beam::laser.k  = V3(0,0,1);
    laserForce.beam::laser.lambda = lambda;
    laserForce.beam::detuning     = detuning;
    laserForce.beam::laser.P_0    = power;
    laserForce.beam::laser.setCollimation(w0);
    laserForce.beam::laser.l = 4;
    laserForce.beam::laser.p = 0;
    laserForce.beam::initialize();
}

double totalIntensity(const LaserForce & laserForce, const Vector<double,3> & r) {
    return laserForce.beam::laser(r);
}

std::ostream & printLaserInfo(const LaserForce & laserForce, std::ostream & out, const std::string & prefix = "") {
    out << prefix << "w0:  " << laserForce.laser.w0 << std::endl;
    out << prefix << "zR:  " << laserForce.laser.zR << std::endl;
    out << prefix << "laser.lambda:  " << laserForce.laser.lambda << std::endl;
    out << prefix << "detuning:  " << laserForce.detuning << std::endl;

    return out;
}

double initial_particle_sigma_x(const LaserForce & laserForce) {
    return laserForce.laser.w0;
}

static const struct {
/* LASER TEST RANGES */
static const double
    xi = -1.0*mm,
    xf =  1.00000001*mm,
    dx = .02*mm,

    yi = -1.0*mm,
    yf =  1.00000001*mm,
    dy = .02*mm,

    zi = -20*cm,
    zf =  20.0000001*cm,
    dz =   5*mm;

static const double Imax = 2e6; /* W/m^2 */
} tr;


#endif // SINGLE_HOLLOW_H
