// -*- c++ -*-
// $Id: trapsim.C 155 2007-07-09 20:00:41Z olsonse $
/*
 * Copyright 2005-2007 Spencer Olson
 *
 * $Log$
 *
 */

/** \file
 * A uniform box with periodic (3D) boundary conditions.  Not too exciting.
 */

static char const trapsim_rcsid[] = "$Id: trapsim.C 155 2007-07-09 20:00:41Z olsonse $";



/* for the laserfield.h file. */
#define DISABLE_SHELL_LOOKUP

#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include <cfloat>

#include <olson-tools/strutil.h>
#include <olson-tools/physical.h>
#include <olson-tools/bfield.h>
#include <olson-tools/random/random.h>
#include <olson-tools/listutil.h>
#include <olson-tools/KeyedBin.h>
#include <olson-tools/GenericBin.h>

#include <dsmc/DSMC.h>
#include <dsmc/ParticleSrc.h>
#include <dsmc/Wall.h>

#include <stdexcept>
#include <sys/times.h>
#include <unistd.h>
#include <signal.h> // so we can issue ^C to gently stop the loading
#include <laser-cooling/StateParticle.h>

#define BIN_VELOCITY 1

static std::vector<std::string> rcsid_vector;
int add_rcsid(std::string rcsid) {
    rcsid_vector.push_back(rcsid);
    return 1;
}

std::string get_rcsid_strings() {
    std::ostringstream ostr;
    ostr << "# ";
    std::copy(rcsid_vector.begin(),
              rcsid_vector.end(),
              std::ostream_iterator<std::string>(ostr, "\n# "));
    ostr << "end of rcsid list";
    return ostr.str();
}

int trapsim_dummy = add_rcsid(trapsim_rcsid);

static bool quit_requested = false;
void quitRequested(int inSignal) {
    quit_requested = true;
}

using namespace physical::constants;
using namespace physical::units;

const double Len = 20*mm;
const double temperature = 10.*uK;

//typedef Particle Particle_t;
typedef StateParticle Particle_t;

typedef octree::VacuumBoundaryObject<Particle_t>     vac_type;

/** Vacuum boundary conditions. */
const int num_vacuo = 6;
vac_type vacuo[num_vacuo] = {
    /*        normal     point            side-length    scaling */
    vac_type(V3(-1,0,0), V3( 0.5*Len,0,0), V3(0,DBL_MAX,DBL_MAX), 0.5*mm),
    vac_type(V3( 1,0,0), V3(-0.5*Len,0,0), V3(0,DBL_MAX,DBL_MAX), 0.5*mm),
#if 1
    vac_type(V3(0,-1,0), V3(0, 0.5*Len,0), V3(DBL_MAX,0,DBL_MAX), 0.5*mm),
    vac_type(V3(0, 1,0), V3(0,-0.5*Len,0), V3(DBL_MAX,0,DBL_MAX), 0.5*mm),
    vac_type(V3(0,0,-1), V3(0,0, 0.5*Len), V3(DBL_MAX,DBL_MAX,0), 0.5*mm),
    vac_type(V3(0,0, 1), V3(0,0,-0.5*Len), V3(DBL_MAX,DBL_MAX,0), 0.5*mm)
#else
    vac_type(V3(0,-1,0), V3(0, 70.*um,0), V3(DBL_MAX,0,DBL_MAX), 0.5*mm),
    vac_type(V3(0, 1,0), V3(0,-70.*um,0), V3(DBL_MAX,0,DBL_MAX), 0.5*mm),
    vac_type(V3(0,0,-1), V3(0,0, 70.*um), V3(DBL_MAX,DBL_MAX,0), 0.5*mm),
    vac_type(V3(0,0, 1), V3(0,0,-70.*um), V3(DBL_MAX,DBL_MAX,0), 0.5*mm)
#endif
};


/* this is just a simple quick and dirty distribution for truncating a normal
 * distrib. */
class TruncatedGaussian3DParticleDistrib {
  public:
    TruncatedGaussian3DParticleDistrib() {
        sigma_x = (1.5*physical::unit::cm);     /* default to 1.5cm */
    }

    void initialize(const Vector<double,3> & origin) {
    }/* initialize */

    inline double x() const {
        return (sigma_x[X]>0.0?
                     std::max(std::min(gauss_deviate(sigma_x[X]), max_x[X]), min_x[X])
                    :0.0);
    }

    inline double y() const {
        return (sigma_x[Y]>0.0?
                     std::max(std::min(gauss_deviate(sigma_x[Y]), max_x[Y]), min_x[Y])
                    :0.0);
    }

    inline double z() const {
        return (sigma_x[Z]>0.0?
                     std::max(std::min(gauss_deviate(sigma_x[Z]), max_x[Z]), min_x[Z])
                    :0.0);
    }

    Vector<double,3>	sigma_x;
    Vector<double,3>	max_x;
    Vector<double,3>	min_x;
};


#ifdef BIN_SPATIAL
KeyedBin<int,double,14,100*100> splineBin(0, 100*100-1);
KeyedBin<int,double,14,100*100> tmpSBin(0, 100*100-1);

template <unsigned int L, class Interpolator>
void sh_spline(const double pos[3],
               Vector<double,L> & retval,
               const std::vector<int> & eltv,
               Interpolator * inp);

#  define P_XYZ   TruncatedGaussian3DParticleDistrib
#  define P_XYZ_TG 1
#  define PV_XYZ /* this is a comment */

#endif

#ifdef BIN_VELOCITY
//#  define P_XYZ   FlatParticleDistrib

#  define P_XYZ   TruncatedGaussian3DParticleDistrib
#  define P_XYZ_TG 1

GenericBin<double,1000> vbin[3] = {
    GenericBin<double,1000>(-100*cm/s,100*cm/s),
    GenericBin<double,1000>(-100*cm/s,100*cm/s),
    GenericBin<double,1000>(-100*cm/s,100*cm/s)
};

GenericBin<double,1000> tmpvbin[3] = {
    GenericBin<double,1000>(-100*cm/s,100*cm/s),
    GenericBin<double,1000>(-100*cm/s,100*cm/s),
    GenericBin<double,1000>(-100*cm/s,100*cm/s)
};

/** Split Gaussian distribution.
*/
class SplitGaussianDistrib {
  public:
    /** Contructor for gaussian distribution.
     * @param b
     *    Set \f$ \beta \f$ for the distribution defined by 
     *    \f$ {\rm e}^{-\beta v^{2}} \f$.
     * @param offs
     *    the center of the Gaussian distribution.
     */
    inline SplitGaussianDistrib(const double & b, const double & offs = 0.0) :
        beta(b), v0(offs) {}
    double beta;
    double v0;

    /** Evaluate \f$ {\rm e}^{-\beta v^{2}} \f$.
     * @param v
     *     Independent variable.
     */
    inline double distrib (const double & v) const {
        if (v < 0) {
            return 0.5; //return exp(-SQR(v+v0) * beta );
        } else {
            return exp(-SQR(v-v0) * beta );
        }
    }
};

class SplitGaussian3Distros {
  public:
    typedef SplitGaussianDistrib P_vx;
    typedef SplitGaussianDistrib P_vy;
    typedef SplitGaussianDistrib P_vz;
};

/** Truncated Gaussian distribution.
*/
class TruncatedGaussianDistrib {
  public:
    /** Contructor for gaussian distribution.
     * @param b
     *    Set \f$ \beta \f$ for the distribution defined by 
     *    \f$ {\rm e}^{-\beta v^{2}} \f$.
     * @param offs
     *    the center of the Gaussian distribution.
     */
    inline TruncatedGaussianDistrib(const double & b, const double & offs = 0.0) :
        beta(b), v0(offs) {}
    double beta;
    double v0;

    /** Evaluate \f$ {\rm e}^{-\beta v^{2}} \f$.
     * @param v
     *     Independent variable.
     */
    inline double distrib (const double & v) const {
        double eta = SQR(v-v0) * beta;
        static const double eta_max  = 0.9; /* trucation done at ~80% in distro tail */
        if (eta > eta_max) {
            return 0.0; /* truncation */
        } else {
            return exp(-eta);
        }
    }
};

class TruncatedGaussian3Distros {
  public:
    typedef TruncatedGaussianDistrib P_vx;
    typedef TruncatedGaussianDistrib P_vy;
    typedef TruncatedGaussianDistrib P_vz;
};

//#  define PV_XYZ ,SplitGaussian3Distros
#  define PV_XYZ ,TruncatedGaussian3Distros


#endif


double Camera_z_min = -100*mm;
double Camera_z_max =  100*mm;
int camera_F2 = 0;
int camera_F3 = 0;
GenericBin<double,1000> camera_bin(-5*mm,4*mm);

void zeroderivs(const double p[VZ+1], const double * time, const double * dt, double rkF[VZ+1], void * ignore);

#include "ConstantLaserField.h"

int main() {
    /* need to be able to add particles to the system. */
    int8_t particle_type = getParticleTypeIndex("85Rb");

    LaserForce laserForce;
    laserForce.super0::mass = particle_info[particle_type].mass;
    initLaser(laserForce);


    const double N              = 1e6; // initial number of MOT atoms
    const double sigma_x        = 300*um; // MOT size

    const double n0             = N / sqrt(M_PI * 2) / sigma_x;
    const double lambda_mfp     = 1.0 / (M_SQRT2 * sigma_x * n0);
    const double Kn             = lambda_mfp / sigma_x;
    /* We'll ensure that dt ~= min(0.1 Coll.Time,0.1 1m/v_avg)  */
    static double dt_max =
        0.1 /* we need to allow excursions 1/10 the size of L */
        * sigma_x /* characteristic length. */
        / sqrt( 8.0*K_B * temperature / pi / particle_info[particle_type].mass);
    const double dt_Kn =
        0.1 /* we need to allow excursions 1/10 the size of L */
        * lambda_mfp /* mean free path. */
        / sqrt( 8.0*K_B * temperature / pi / particle_info[particle_type].mass);
    const double dt = std::min(dt_Kn, dt_max);
    static double t_max = 2.*s + dt /* dt * 200.0 */;
    static int spit_out_data = 0;

    const double desired_samples = 115;
    const double sample_wait_time = 100.0*s; /* time before real sampling happens. */
    static int sample_interval = int( (t_max/*-sample_wait_time*/) / ( desired_samples * dt ) );
    static int write_interval = 2783 ;//* sample_interval;


    ParticleSrc< P_XYZ PV_XYZ, Particle_t > atomsrc;
    atomsrc.seed.type = particle_type;
    atomsrc.seed.dt   = 10*us;
    //atomsrc.state = StateParticle::F2;

#ifdef P_XYZ_TG
    atomsrc.pos_distrib.sigma_x         = sigma_x;
    atomsrc.pos_distrib.max_x           = 4*sigma_x;
    atomsrc.pos_distrib.min_x           =-4*sigma_x;
#else
    atomsrc.pos_distrib.delta_x         = 0.4999*Len;
#endif

    atomsrc.flux                        = N / dt;

    //particle_info[particle_type].size   = 1;
    particle_info[particle_type].size   = atomsrc.flux * 1e-3 * dt;
    atomsrc.pulsed_source               = true;
    atomsrc.pulse_duration              = dt;
    atomsrc.origin                      = V3(0,0,0);
    atomsrc.temperature                 = 1e-5*temperature;
    atomsrc.velocity                    = 0.0; //200.0*m/s;
    //atomsrc.velocity[X]                 = 20.0*cm/s;
    atomsrc.initialize();


    std::cout << get_rcsid_strings()                            << '\n'
              << "(" << getpid() << ")  t_max address:  " << (&t_max) << '\n'
              << "(" << getpid() << ")  spit_out_data address:  " << (&spit_out_data) << '\n'
              << "(" << getpid() << ")  sample_interval address:  " << (&sample_interval) << '\n'
              << "(" << getpid() << ")  write_interval address:  " << (&write_interval) << '\n'
              << "(" << getpid() << ")  write interval (s) :  " << (write_interval*dt) << '\n'
              << "(" << getpid() << ")  dt:  " << dt << '\n'
              << "(" << getpid() << ")  N :  " << N << '\n'
              << "(" << getpid() << ")  n0:  " << n0 << '\n'
              << "(" << getpid() << ")  FN:  " << particle_info[particle_type].size << std::endl;
    printLaserInfo(laserForce, std::cout, "(" + to_string(getpid()) + ")  ");

    DSMC< RK5AdaptiveIntegrator< StatisticalForceRKWrapper<LaserForce> >,
          octree::CMAndVolNotShrink,
          NULLDXHook,
          Particle_t > dsmc;
    //dsmc.integrator.derivs = (derivativesFunction)Derivs<LaserForce>::derivs;
    dsmc.integrator.derivs = zeroderivs;
    dsmc.integrator.errmax = 1e-6;
    dsmc.integrator.rkTweak.statisticalForcePtr = &laserForce;
    dsmc.allow_collisions = false;

    for (int i = 0; i < num_vacuo; i++) {
        dsmc.addBoundaryObject(&vacuo[i]);
    }

    dsmc.data_radius_of_influence = 4;
    dsmc.minimum_octree_particles = 20;

    /* ** plotting all particles. */
    std::ofstream fout("particles.plt");
    std::ofstream fdata("data.plt");
    std::ofstream fpout("data.dat");

    std::ofstream fcout("collision-totals.dat");
    fout << "set xlabel 'x'\nset ylabel 'y'\nset zlabel 'z'\n";
    std::string tree_plotopt = "w lp ps 1 t ''" ;
    std::string part_plotopt = "w p pt 6 ps 2 lt 2 t 'particles'";
    std::ostringstream ostr;


    fcout << "#  Collisions Totals\n"
             "#  Sim. Time(s)        inst. coll./s"
             "          avg. coll./s"
             "          avg. coll. selection rate (per atom)"
             "          Num. Super Particles"
             "          Cycle CPU Time(s)\n";

    std::ofstream splinebinout("data_spline.dat");

    std::ofstream kbinout("data_binned.dat");


    std::ofstream vbinout[3] = {
        std::ofstream("vbinX.dat"),
        std::ofstream("vbinY.dat"),
        std::ofstream("vbinZ.dat")
    };

    std::ofstream scatout("scatter.dat");
    scatout.precision(10);

    std::ofstream cameraout("camera.dat");

    std::ofstream state_lifetime_camera("state_lifetime_camera.dat");
    std::ofstream total_scattering_fout("total_scattering.dat");
    total_scattering_fout << "# time\tN_tests\tN_scat\tdt_tot" << std::endl;

    double max_dx[3] = {0.0, 0.0, 0.0};

    /* now let the dsmc have some fun with the particles. */
    try {
        signal(SIGINT, quitRequested);
        double seconds_per_clock_tick = 1.0 / sysconf(_SC_CLK_TCK);
        while ( !quit_requested && dsmc.getTotalTime() <= t_max) {
            dsmc.addParticles(atomsrc.createParticles(dt));

            struct tms ti, tf;
            times(&ti);
            dsmc.evolve(dt, &laserForce);
            times(&tf);

            if (dsmc.getTotalTime() >= sample_wait_time) spit_out_data = 1;

            {
                double cycle_cpu_time = 
                     ( ( (tf.tms_utime + tf.tms_stime) - (ti.tms_utime + ti.tms_stime) )
                       * seconds_per_clock_tick
                     );

                /* plot collision totals every time-step. */
                fcout << dsmc.getTotalTime()       << '\t'
                      << (dsmc.getCycleCollisions()/(dt*dsmc.numberParticles())) << '\t'
                      << (dsmc.getTotalCollisions()/(dsmc.getTotalTime()*dsmc.numberParticles())) << '\t'
                      /* the following is supposed to be collision selection rate
                       * per atom.  That is why the factor of 2 precedes the line.
                       */
                      << (2*dsmc.getTotalCollisionSelections()/(dsmc.getTotalTime()*dsmc.numberParticles())) << '\t'
                      << dsmc.numberParticles() << '\t'
                      << cycle_cpu_time << std::endl;
            }

            if ((int(dsmc.getTotalTime() / dt) - 1) % sample_interval  == 0) {

#ifdef BIN_SPATIAL

                /* instead of just writing out the data, we'll bin it and
                 * then write out the binned data.
                 */
                const int ELT_LEN=11;
                const int ELT[ELT_LEN] = {
/* 6 */             DSMCData<>::NUMBER_DENSITY,
/* 7 */             DSMCData<>::NUMBER,
/* 8 */             DSMCData<>::MAX_SIGMA_RELSPEED,
/* 9 */             DSMCData<>::NUMBER_COLLISION_PAIRS_REMAINDER,
/* 10*/             DSMCData<>::NUMBER_COLLISIONS,
/* 11*/             DSMCData<>::AVG_V_X,
/* 12*/             DSMCData<>::AVG_V_Y,
/* 13*/             DSMCData<>::AVG_V_Z,
/* 14*/             DSMCData<>::AVG_V2_X,
/* 15*/             DSMCData<>::AVG_V2_Y,
/* 16*/             DSMCData<>::AVG_V2_Z
                };
                const std::vector<int> eltv(ELT, ELT+ELT_LEN);

                int row_len = (int)sqrt(splineBin.nBins());
                const double dx = 1.0 / ((double)row_len - 1.0);
                for (unsigned int i = 0; i < splineBin.nBins(); i++) {
                    Vector<double,14> v;
                    Vector<double,11> & v11 = *((Vector<double,11>*)(&v.val[3]));
                    Vector<double,3> & x = V3C(v.val);

                    /* we'll assume that vol_dx[X] >= vol_dx[Y] */
                    x[X] = (i / row_len) * dx - 0.5*Len;
                    x[Y] = (i % row_len) * dx - 0.5*Len;
                    x[Z] = 0.0; /* we're sampling in the middle of the field */

                    v11 = 0.0;
                    sh_spline(x.val, v11, eltv, dsmc.getInterpolator());
                    splineBin.bin(i, v);
                }

#endif
#ifdef BIN_VELOCITY

                for (int i = 0; i < dsmc.numberParticles(); i++) {
                    Vector<double,3> & v = V3C(dsmc.getParticles()[i]->vel);
                    vbin[X].bin(v[X]);
                    vbin[Y].bin(v[Y]);
                    vbin[Z].bin(v[Z]);
                }
#endif


            }

            /* **** TAKE CARE OF PLOTTING ALL PARTICLES. */
            if (dsmc.getTotalTime() >= (t_max-dt) || (int(dsmc.getTotalTime() / dt) - 1) % write_interval  == 0) {
                std::string tot_time = to_string(dsmc.getTotalTime());
#if 1
                /* note that this octree is not necessarily representative of the
                 * current particles.  This is because there has been a call to
                 * dsmc.evolve since the Octree was built.
                 */
#if 0
                dsmc.plotParticlesAndTree(fout,
                                          tot_time + " s",
                                          tree_plotopt,
                                          part_plotopt)
    	            << "pause p\n" << std::flush;
#endif
                dsmc.printParticles(fout) << "\n\n" << std::flush;
#endif

                camera_F2 = camera_F3 = 0;
                camera_bin.clearBins();
                for (int i = 0; i < dsmc.numberParticles(); i++) {
                    StateParticle & p = *dsmc.getParticles()[i];
                    if (p.pos[Z] >= Camera_z_min && p.pos[Z] <= Camera_z_max) {
                        if(p.state == StateParticle::F2) camera_F2++;
                        else camera_F3++;

                        /* The camera integrates in the Y direction. */
                        camera_bin.bin(p.pos[X]);
                    }
                }

                state_lifetime_camera
                    << tot_time << '\t'
                    << camera_F2 << '\t'
                    << camera_F3 << std::endl;

                camera_bin.print(cameraout, tot_time + "\t") << std::endl;

                //SScatBin *= (1.0/(write_interval * dt));
                SScatBin.print(scatout);
                SScatBin.clearBins();

                total_scattering_fout
                    << tot_time << '\t'
                    << total_scatter_tests << '\t'
                    << total_scattering << '\t'
                    << total_scattering_time << std::endl;
                total_scatter_tests = 0;
                total_scattering = 0;
                total_scattering_time = 0.0;

                if (dsmc.getInterpolator()) {
                    /* ok, let's spit out the data tree. */
#if 0
                    dsmc.getInterpolator()->
                         plotParticlesAndTree(fdata,
                                              "DSMC Data Tree:  " + tot_time + " s",
                                              tree_plotopt,
                                              part_plotopt)
                     << "pause -1\n" << std::flush;
#endif

#if 0
                    fpout << "#  Data Particles\n#  Time:  " << tot_time << " s\n";
                    dsmc.getInterpolator()->printParticles(fpout) << "\n\n";
#endif

#ifdef BIN_SPATIAL

                    /* write out spline data. */
                    tmpSBin = splineBin;
                    /* take average. */
                    for (unsigned int i = 0; i < tmpSBin.nBins(); i++) {
                        if (tmpSBin.hist[i]) tmpSBin.bins[i] /= tmpSBin.hist[i];
                    }

                    /* write out new shmooshed together spline data. */
                    splinebinout << "#  Averaged bin across channel\n#  Time:  " << tot_time << " s\n";
                    tmpSBin.print(splinebinout,"") << "\n\n" << std::flush;
#endif

#ifdef BIN_VELOCITY
                    /* We're going to normalize the bin before we write it
                     * out. */
                    int vbinsum[3] = {0,0,0};
                    for (unsigned int i = 0; i < vbin[X].nBins(); i++) {
                        vbinsum[X] += vbin[X].bins[i];
                        vbinsum[Y] += vbin[Y].bins[i];
                        vbinsum[Z] += vbin[Z].bins[i];
                    }

                    /* the factor of 1e7 is just to make sure that we have a
                     * dynamic range of 0:1e4 after converting back to
                     * integer. (Remember that there are 1000 bins in the
                     * histogram.) */
                    for (unsigned int i = 0; i < vbin[X].nBins(); i++) {
                        tmpvbin[X].bins[i] = (int)rint( 1e7 * ((double)vbin[X].bins[i]) / ((double)vbinsum[X]) );
                        tmpvbin[Y].bins[i] = (int)rint( 1e7 * ((double)vbin[Y].bins[i]) / ((double)vbinsum[Y]) );
                        tmpvbin[Z].bins[i] = (int)rint( 1e7 * ((double)vbin[Z].bins[i]) / ((double)vbinsum[Z]) );
                    }

                    /* write out new shmooshed together spline data. */
                    vbinout[X] << "#  Normalized velocity histogram\n#  Time:  " << tot_time << " s\n";
                    vbinout[Y] << "#  Normalized velocity histogram\n#  Time:  " << tot_time << " s\n";
                    vbinout[Z] << "#  Normalized velocity histogram\n#  Time:  " << tot_time << " s\n";
                    tmpvbin[X].print(vbinout[X],tot_time + '\t') << "\n\n" << std::flush;
                    tmpvbin[Y].print(vbinout[Y],tot_time + '\t') << "\n\n" << std::flush;
                    tmpvbin[Z].print(vbinout[Z],tot_time + '\t') << "\n\n" << std::flush;
#endif

                    if ( !spit_out_data ) {
                        /* we'll just keep clearing the bins until we need to
                         * really start time-averaging.
                         */
#ifdef BIN_SPATIAL
                        splineBin.clearBins();
#endif


#ifdef BIN_VELOCITY
                        vbin[X].clearBins();
                        vbin[Y].clearBins();
                        vbin[Z].clearBins();
#endif
                    }

                    /* now let's zero the collisions-per-octree-node counters
                     * so that we only get number collisions per sample time.
                     */
                    DSMCData<Particle_t> ** d = dsmc.getInterpolator()->getParticles();
                    for (int i = dsmc.getInterpolator()->numberParticles() - 1; i >= 0; i--) {
                        d[i]->data.name.collisions = 0;
                    }
                } else if (dsmc.numberParticles() > 0 && spit_out_data) {
                    /* we have to make our own interpolator since DSMC didn't need
                     * to construct it. */
                    octree::OctreeInterpolator<DSMCData<Particle_t>,Particle_t> ointerp;
                    DSMCData<Particle_t>::InitInfo init = {dsmc.getParticles()};

                    ointerp.createDataOctree(dsmc.getOctree(), init);
                    ointerp.setRadiiOfInfluence(dsmc.data_radius_of_influence);

                    fpout << "#  Data Particles\n#  Time:  " << tot_time << " s\n";
                    ointerp.printParticles(fpout) << "\n\n";
                }

            }/* if */

            for (int i = X; i <= Z; i++)
                max_dx[i] = std::max(max_dx[i], dsmc.max_particle_transit(i));
        }/* while */
    } catch (std::exception & e) {
        std::cout << "exception:  " << e.what() << std::endl;
    }

    fout.close();
    fpout.close();
    fdata.close();
    fcout.close();

    vbinout[X].close();
    vbinout[Y].close();
    vbinout[Z].close();

    std::cout << "max Particle Transit:  ("
              << max_dx[X] << ','
              << max_dx[Y] << ','
              << max_dx[Z] << ')'
              << std::endl;

    return 0;
}

template <unsigned int L, class Interpolator>
void sh_spline(const double pos[3],
               Vector<double,L> & retval,
               const std::vector<int> &eltv,
               Interpolator * inp) {
    if (!inp) return;
    if (L != eltv.size()) {
        throw std::runtime_error("elt vector != size of return value!");
    }

    std::vector<double> interpolant = inp->interpolate(pos, eltv);

    if (interpolant.size() == 0) {
        retval = 0.0;
    } else {
        for (unsigned int i = 0; i < L; i++) {
            retval[i] = interpolant[i];
        }
    }
}

void zeroderivs(const double p[VZ+1], const double * time, const double * dt, double rkF[VZ+1], void * ignore) {
    /* we're going to leave this a 1d problem to compare to Birds code. */
    rkF[X] = 0.0;
    rkF[Y] = 0.0;
    rkF[Z] = 0.0;

    /* no forces. */
    rkF[VX] = 0.0;
    rkF[VY] = 0.0;
    rkF[VZ] = 0.0;
}

