// -*- c++ -*-
// $Id$
/** \file
 * Factorial function that uses a cache to speed up evalulation.
 * This file also includes a generalized Laguerre function.
 *
 * Copyright 2004-2007 Spencer Olson
 *
 */


#ifndef FACTORIAL_CACHE_H
#define FACTORIAL_CACHE_H



static double * f = NULL; 
static long f_size = 0;


inline double fact(const int & k) {
    if (k >= f_size) {
        size_t f_size_old = f_size;
        f_size = k + 1;

        /* begin futher allocation for factorial lookup table */
        f = (double*)realloc(f, f_size * sizeof(double));

        //printf("realloc()'d f[] to length = %i from %i\n", (int)f_size, (int)f_size_old);

        if (f_size_old == 0) {
            f[0] = 1.0;
            f_size_old = 1;
        }

        for (int i = f_size_old; i <= k; i++) {
            f[i] = f[i-1]*i;
        }

    }//fi

    return f[k];
}

/**

% function genL = LaguerreL(l,p,rho)
%       returns the general Laguerre function L(l,p,rho)
%       where:
%               l = azimuthal order
%               p = radial order
%               rho = radial position
*/
inline double LaguerreL(const int & l, const int & p, const double & rho) {
    double L = 0;
    double minus_1_raised_to_m = -1;

    for (int m = 0; m <= p; m++) {
        minus_1_raised_to_m *= -1;

        L += minus_1_raised_to_m
           / (fact(p-m) * fact(abs(l)+m) * fact(m))
           * fast_pow(rho,m);
    }

    L *= fact(abs(l)+p);
    return L;
}



#endif // FACTORIAL_CACHE_H
