// -*- c++ -*-
// $Id$
/*
 * Copyright 2004-2007 Spencer Olson
 *
 */

/*
 * Unformatted versions of equations below :
 * E-field
 *   E(rho) = E_0 * exp( -rho^2 / w^2) * ( w0 / w )
 *
 * "Beam waist"
 *   w(z)   = w0 * sqrt( 1 + ( lambda z / (pi w0^2) )^2 )
 *
 * Radius of curvature
 *   R(z)   = z * ( 1 + ( pi w0^2 / (z lambda) )^2 )
 *   (not needed in our approximation)
 *
 * Power of beam :
 *   P = (I_0 /2) pi w0^2
 */

/** \file
 * Information to describe laser beams.
 * Some of the semantics and algorithms in this code is taken from Georg's
 * code that simulated Kevin's experiment.
 *
 * E-field
 *   \f$ E(\rho) = E_0 e^\left( \frac{-\rho^2} {w^2} \right) \frac{w_0}{w} \f$
 *
 * "Beam waist"
 *   \f$ w(z)   = w_0 \sqrt{ 1 + \left( \frac{\lambda z} {\pi w_0^2} \right)^2 } \f$
 *
 * Radius of curvature
 *   \f$ R(z)   = z \left( 1 + \left( \frac{\pi w_0^2} {z \lambda} \right)^2 \right) \f$
 *   (not needed in our approximation)
 *
 *

 * +++++++++++++  Polarization conventions ++++++++++++++++++++++
 *
 * I FOUND: A beam with pure sigma-polarization of one kind travels
 *          at a direction phi wrt the local B-direction.
 *          The intensity of the beam is I0, Then, in the
 *          frame of the local B-direction the intensities are
 *
 *        pi                  : 0.5 * I0 * sin^2(phi)
 *        sigma of same kind  : I0 * cos^4(phi/2)
 *        sigma of other kind : I0 * sin^4(phi/2)
 *
 *        Expression seems correct over full range of phi (0 to pi).
 *

 * Beam polarizations: All beams are either left- or right-handed
 * circularly polarized. We use the following conventions:
 *
 *           wavevector k parallel B
 *           atom becomes excited into state m=+1 (low-field-seeking)
 *              --> polarization +1
 *
 *           wavevector k parallel B
 *           atom becomes excited into state m=-1 (high-field-seeking)
 *              --> polarization -1
 *
 *           wavevector k anti-parallel B
 *           atom becomes excited into state m=+1 (low-field-seeking)
 *              --> polarization -1
 *
 *           wavevector k anti-parallel B
 *           atom becomes excited into state m=-1 (high-field-seeking)
 *              --> polarization +1
 */

#ifndef LASER_H
#define LASER_H

#include <olson-tools/physical.h>
#include <olson-tools/Vector.h>
#include <olson-tools/Fields.h>
#include <cfloat>
#include "FactorialCache.h"

#include "GaussianLaser.h"
#include "LaguerreGaussianLaser.h"
#include "laserfield-lookup.h"

/* Lasers are a part of the LaserCooling namespace. */

#endif // LASER_H
