// -*- c++ -*-
// $Id$
/** \file
 * Scale an intensity source.
 *
 * Copyright 2004-2007 Spencer Olson
 *
 */



#ifndef SCALE_INTENSITY_H
#define SCALE_INTENSITY_H

namespace LaserCooling {

template <class IntensityField>
class ScaleIntensity : public virtual BaseField, public IntensityField {
  public:
    typedef BaseField      super0;
    typedef IntensityField super1;

    ScaleIntensity () : super0(), super1(), scale(1.0), t_i(0.0), t_f(-0.5) {}

    /** Compute/lookup the intensity of an optical source.
     * @param r
     *     The position to evaluate the field.
     */
    inline double operator()(const Vector<double,3> & r) const {
        return  scale * super1::operator()(r);
    }

    inline void set_scale(const double & t) {
        if (t <= t_i) {
            scale = 0.0;
        } else if (t >= t_f) {
            scale = 1.0;
        } else {
            scale = (t - t_i) / (t_f - t_i);
        }
    }

    double scale;
    double t_i, t_f;
};

}

#endif //SCALE_INTENSITY_H
