// -*- c++ -*-
/* $Id$
 *
 * Copyright 2004-2007 Spencer Olson
 *
 */

/** \file Program to show the statistical distributions resulting from dipole
 * scattering.
*/


#include <math.h>
#include <stdlib.h>
#include <olson-tools/physical.h>
#include <olson-tools/random/random.h>
#include <iostream>
#include <fstream>
#include <olson-tools/Distribution.h>
#include <olson-tools/power.h>
#include <olson-tools/Vector.h>
#include <olson-tools/GenericBin.h>
#include <olson-tools/indices.h>

using namespace physical::constants;
using std::cin;
using std::cout;
using std::endl;
using std::flush;

/** A flat distribution for use.
*/
struct {
    /** Return 0.5.
     */
    inline double distrib (const double & x) const {
        return 0.5;
    }
} FlatDistribution;

struct {
    /** Return 0.5.
     */
    inline double distrib (const double & x) const {
        return SQR(cos(x));
    }
} P_THETA;

/* get the speed from a maxwellian distribution. */
const double beta = 1.0;
const double sigma = sqrt(0.5/beta);

static Distribution p_theta = Distribution(P_THETA, 0.0, pi, 10000);
static Distribution p_phi   = Distribution(FlatDistribution, 0.0, 2*pi, 10000);

typedef GenericBin<double,1000> bin;

int main() {

    int niter = 0;
    cout << "Enter the number of iterations to perform  : " << flush;
    cin >> niter;
    if (niter <= 0) {
        cout << "niter ("<<niter<<") out of range" << endl;
        exit(EXIT_FAILURE);
    }

    int walks = 0;
    cout << "Enter the number of walks to perform  : " << flush;
    cin >> walks;
    if (walks <= 0) {
        cout << "walks ("<<walks<<") out of range" << endl;
        exit(EXIT_FAILURE);
    }

    const int nbins = 8;
    bin bins[nbins] = {
        bin(0, 4*sqrt(walks)),
        bin(-4*sqrt(walks), 4*sqrt(walks)),
        bin(-4*sqrt(walks), 4*sqrt(walks)),
        bin(-4*sqrt(walks), 4*sqrt(walks)),
        bin(-1, 1),
        bin(-1, 1),
        bin(0, pi),
        bin(0, 2*pi)
    };

    for (int i = 0; i < niter; i++) {
        Vector<double,3> x = V3(0,0,0);
        for (int k = 0; k < walks; k++) {
            double theta = p_theta();
            double phi   = p_phi();
            bins[6].bin(theta);
            bins[7].bin(phi);

            Vector<double,3> dx = V3(cos(phi)*sin(theta), sin(phi)*sin(theta), cos(theta));

            x += dx;
        }

        double xlen = x.abs();
        bins[0].bin(xlen);
        bins[1].bin(x[X]);
        bins[2].bin(x[Y]);
        bins[3].bin(x[Z]);

        double costheta = x[Z]/xlen;
        double sintheta = sqrt(1 - SQR(costheta));
        double cosphi = 1, sinphi = 0;
        if (sintheta) {
            cosphi = x[X] / hypot( x[X], x[Y]);
            sinphi = x[Y] / hypot( x[X], x[Y]);
        }

        bins[4].bin(cosphi);
        bins[5].bin(costheta);
    }

    std::ofstream outf("dist.dat");
    for (int i = 0; i < nbins; i++) {
        bins[i].print(outf,"") << "\n\n";
    }
    outf.close();

    return 0;
}

