// -*- c++ -*-
// $Id$
/*
 * Copyright 2000-2007 Spencer Olson
 *
 */

#include <math.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>

#include <olson-tools/physical.h>
#include <olson-tools/random/random.h>
#include <olson-tools/Distribution.h>
#include <olson-tools/Vector.h>
#include <olson-tools/GenericBin.h>
#include <olson-tools/indices.h>

using namespace physical::constants;

/** A flat distribution for use.
*/
typedef struct {
    /** Return 0.5.
     */
    inline double distrib (const double & x) const {
        return 0.5;
    }
} FlatDistribution;

typedef struct {
    inline double distrib (const double & x) const {
        return SQR(cos(x));
    }
} P_THETA;

static Distribution p_theta = Distribution(P_THETA(), 0, pi, 10000);
static Distribution p_phi   = Distribution(FlatDistribution(), 0, 2*pi, 10000);

typedef GenericBin<double,1000> bin;

int main() {

    int iter = 0;
    std::cout << "Enter the nubmer of samples:  "
              << std::flush;
    std::cin >> iter;
    if (iter == 0) return EXIT_FAILURE;
    std::cout << iter << " samples requested." << std::endl;

    int walks = 0;
    std::cout << "Enter the nubmer of walks:  "
              << std::flush;
    std::cin >> walks;
    if (walks == 0) return EXIT_FAILURE;
    std::cout << walks << " walks requested." << std::endl;

    const int nbins = 6;
    bin bins[nbins] = {
        bin(0, 10*sqrt(walks/2.0)),
        bin(-10*sqrt(walks/2.0), 10*sqrt(walks/2.0)),
        bin(-10*sqrt(walks/2.0), 10*sqrt(walks/2.0)),
        bin(-10*sqrt(walks/2.0), 10*sqrt(walks/2.0)),
        bin(0, pi),
        bin(0, 2*pi)
    };

    for (int i = 0; i < iter; i++) {
        Vector <double,3> x = V3(0,0,0);

        for (int j = 0; j < walks; j++) {
            double phi   = p_phi();
            double theta = p_theta();
            //Vector<double,3> dx = V3(cos(theta)*sin(phi), sin(theta)*sin(phi), cos(phi));
            Vector<double,3> dx = V3(cos(phi)*sin(theta), sin(phi)*sin(theta), cos(theta));
            x += dx;

            bins[4].bin(theta);
            bins[5].bin(phi);
        }

        bins[0].bin(x.abs());
        bins[1].bin(x[X]);
        bins[2].bin(x[Y]);
        bins[3].bin(x[Z]);
    }

    std::ofstream outf("dist.dat");
    for (int j = 0; j < nbins; j++) {
        bins[j].print(outf,"") << "\n\n";
    }

    return 0;
}

